@@ -1,4 +1,5 @@
 import logging
+import importlib
 from warnings import warn
 from typing import Any, Dict, List, Optional, Tuple, Type, Union
 
@@ -11,6 +12,7 @@ from skfem.mapping import Mapping
 from skfem.mesh import Mesh
 from skfem.quadrature import get_quadrature
 from skfem.refdom import Refdom
+from skfem.generic_utils import deprecated
 
 
 logger = logging.getLogger(__name__)
@@ -22,7 +24,7 @@ class AbstractBasis:
     Please see the following implementations:
 
     - :class:`~skfem.assembly.CellBasis`, basis functions inside elements
-    - :class:`~skfem.assembly.ExteriorFacetBasis`, basis functions on boundary
+    - :class:`~skfem.assembly.FacetBasis`, basis functions on boundary
     - :class:`~skfem.assembly.InteriorFacetBasis`, basis functions on facets
       inside the domain
 
@@ -30,13 +32,14 @@ class AbstractBasis:
 
     mesh: Mesh
     elem: Element
+    mapping: Mapping
     tind: Optional[ndarray] = None
+    tind_normals: Optional[ndarray] = None
     dx: ndarray
     basis: List[Tuple[DiscreteField, ...]] = []
     X: ndarray
     W: ndarray
     dofs: Dofs
-    _side: int = 1
 
     def __init__(self,
                  mesh: Mesh,
@@ -116,10 +119,10 @@ class AbstractBasis:
             D = tuple(D[0][key].all() for key in D[0])
         return np.setdiff1d(np.arange(self.N), np.concatenate(D))
 
+    @deprecated("Basis.get_dofs")
     def find_dofs(self,
                   facets: Dict[str, ndarray] = None,
                   skip: List[str] = None) -> Dict[str, DofsView]:
-        warn("find_dofs deprecated in favor of get_dofs.", DeprecationWarning)
         if facets is None:
             if self.mesh.boundaries is None:
                 facets = {'all': self.mesh.boundary_facets()}
@@ -239,55 +242,15 @@ class AbstractBasis:
             raise ValueError
 
         if elements is not None:
-            elements = self._normalize_elements(elements)
+            elements = self.mesh.normalize_elements(elements)
             return self.dofs.get_element_dofs(elements, skip_dofnames=skip)
 
-        facets = self._normalize_facets(facets)
+        facets = self.mesh.normalize_facets(facets)
         return self.dofs.get_facet_dofs(facets, skip_dofnames=skip)
 
-    def _normalize_facets(self, facets):
-        if isinstance(facets, ndarray):
-            return facets
-        if facets is None:
-            return self.mesh.boundary_facets()
-        elif isinstance(facets, (tuple, list, set)):
-            return np.unique(
-                np.concatenate(
-                    [self._normalize_facets(f) for f in facets]
-                )
-            )
-        elif callable(facets):
-            return self.mesh.facets_satisfying(facets)
-        elif isinstance(facets, str):
-            if ((self.mesh.boundaries is not None
-                 and facets in self.mesh.boundaries)):
-                return self.mesh.boundaries[facets]
-            else:
-                raise ValueError("Boundary '{}' not found.".format(facets))
-        raise NotImplementedError
-
-    def _normalize_elements(self, elements):
-        if isinstance(elements, ndarray):
-            return elements
-        if callable(elements):
-            return self.mesh.elements_satisfying(elements)
-        elif isinstance(elements, (tuple, list, set)):
-            return np.unique(
-                np.concatenate(
-                    [self._normalize_elements(e) for e in elements]
-                )
-            )
-        elif isinstance(elements, str):
-            if ((self.mesh.subdomains is not None
-                 and elements in self.mesh.subdomains)):
-                return self.mesh.subdomains[elements]
-            else:
-                raise ValueError("Subdomain '{}' not found.".format(elements))
-        raise NotImplementedError
-
     def __repr__(self):
         size = sum([sum([y.size if hasattr(y, 'size') else 0
-                         for y in x])
+                         for y in x.astuple])
                     for x in self.basis[0]]) * 8 * len(self.basis)
         rep = ""
         rep += "<skfem {}({}, {}) object>\n".format(type(self).__name__,
@@ -322,15 +285,18 @@ class AbstractBasis:
         if w.shape[0] != self.N:
             raise ValueError("Input array has wrong size.")
 
-        refs = self.basis[0]
+        if isinstance(self.elem, ElementVector):
+            # ElementVector shouldn't get split here: workaround
+            refs: List[Tuple[ndarray, 'AbstractBasis']] = [(np.array([]),
+                                                            self)]
+        else:
+            refs = self.split(w)
         dfs: List[DiscreteField] = []
 
         # loop over solution components
         for c in range(len(refs)):
-            ref = refs[c]
-            if ref.is_zero():
-                dfs.append(ref)
-                continue
+            ref = refs[c][1].basis[0][0]
+            ref = ref.astuple
             fs = []
 
             def linear_combination(n, refn):
@@ -338,10 +304,8 @@ class AbstractBasis:
                 out = 0. * refn.copy()
                 for i in range(self.Nbfun):
                     values = w[self.element_dofs[i]]
-                    if self.basis[i][c].is_zero():
-                        continue
                     out += np.einsum('...,...j->...j', values,
-                                     self.basis[i][c][n])
+                                     self.basis[i][c].get(n))
                 return out
 
             # interpolate DiscreteField
@@ -382,7 +346,7 @@ class AbstractBasis:
                                e.facet_dofs,
                                e.interior_dofs])
             return output
-        raise ValueError("Basis.elem has only a single component!")
+        return [np.unique(self.dofs.element_dofs)]
 
     def split_bases(self) -> List['AbstractBasis']:
         """Return Basis objects for the solution components."""
@@ -394,7 +358,7 @@ class AbstractBasis:
             return [type(self)(self.mesh, self.elem.elem, self.mapping,
                                quadrature=self.quadrature)
                     for _ in range(self.mesh.dim())]
-        raise ValueError("AbstractBasis.elem has only a single component!")
+        return [self]
 
     @property
     def quadrature(self):
@@ -418,21 +382,59 @@ class AbstractBasis:
         """Create a copy of ``self`` that uses different element."""
         raise NotImplementedError
 
+    def global_coordinates(self) -> ndarray:
+        raise NotImplementedError
+
+    def _normalize_interp(self, interp) -> Tuple[ndarray, ...]:
+
+        if isinstance(interp, ndarray):
+            pass
+        elif callable(interp):
+            interp = interp(self.global_coordinates())
+        elif isinstance(interp, (float, int)):
+            interp = interp + self.zero_w()
+        elif isinstance(interp, (tuple, list)):
+            interp = tuple(self._normalize_interp(c) for c in interp)
+
+        return interp
+
     def _projection(self, interp):
 
         from skfem.assembly import BilinearForm, LinearForm
         from skfem.helpers import inner
 
-        if isinstance(interp, float):
-            interp = interp + self.global_coordinates().value[0] * 0.
-
-        if callable(interp):
-            interp = interp(self.global_coordinates().value)
+        interp = self._normalize_interp(interp)
+        if not isinstance(interp, tuple):
+            interp = (interp,)
+        assert len(interp) == len(self.basis[0])
 
         return (
-            BilinearForm(lambda u, v, _: inner(u, v)).assemble(self),
-            LinearForm(lambda v, w: inner(interp, v)).assemble(self),
+            (BilinearForm(lambda *args: inner(args[:(len(args) - 1) // 2],
+                                              args[(len(args) - 1) // 2:-1]))
+             .assemble(self)),
+            (LinearForm(lambda *args: inner(interp,
+                                            args[:(len(args) - 1)]))
+             .assemble(self)),
         )
 
     def project(self, interp, **kwargs):
         raise NotImplementedError
+
+    def plot(self, x, visuals='matplotlib', **kwargs):
+        """Convenience wrapper for skfem.visuals."""
+        mod = importlib.import_module('skfem.visuals.{}'.format(visuals))
+        return mod.plot(self, x, **kwargs)
+
+    def plot3(self, x, visuals='matplotlib', **kwargs):
+        """Convenience wrapper for skfem.visuals."""
+        if not isinstance(visuals, str):
+            logger.warning("Second argument, 'visuals', must be a string.")
+        mod = importlib.import_module('skfem.visuals.{}'.format(visuals))
+        return mod.plot3(self, x, **kwargs)
+
+    def draw(self, visuals='matplotlib', **kwargs):
+        """Convenience wrapper for skfem.visuals."""
+        if not isinstance(visuals, str):
+            logger.warning("First argument, 'visuals', must be a string.")
+        mod = importlib.import_module('skfem.visuals.{}'.format(visuals))
+        return mod.draw(self, **kwargs)
