@@ -1,5 +1,5 @@
 from .element_line_p0 import ElementLineP0  # noqa
-from .element_line_p1 import ElementLineP1  # noqa
+from .element_line_p1 import ElementLineP1, ElementLineP1DG  # noqa
 from .element_line_p2 import ElementLineP2  # noqa
 from .element_line_pp import ElementLinePp  # noqa
 from .element_line_hermite import ElementLineHermite  # noqa
