@@ -5,7 +5,7 @@ so normally the user is not required to initialize these classes.
 
 """
 
-from .mapping import Mapping
-from .mapping_affine import MappingAffine
-from .mapping_isoparametric import MappingIsoparametric
-from .mapping_mortar import MappingMortar
+from .mapping import Mapping  # noqa
+from .mapping_affine import MappingAffine  # noqa
+from .mapping_isoparametric import MappingIsoparametric  # noqa
+from .mapping_mortar import MappingMortar  # noqa
