@@ -16,6 +16,7 @@ Default constructor creates a mesh for the unit square:
   Number of elements: 2
   Number of vertices: 4
   Number of nodes: 4
+  Named boundaries [# facets]: left [1], bottom [1], right [1], top [1]
 
 Each mesh type has several constructors; see the docstring, e.g.,
 ``help(MeshTri)`` or click :class:`~skfem.mesh.MeshTri` in the online
@@ -70,6 +71,10 @@ class MeshLineConstructor:
     def __getattr__(self, name):
         return getattr(MeshLine1, name)
 
+    @classmethod
+    def __instancecheck__(_, instance):
+        return isinstance(instance, MeshLine1)
+
 
 MeshLine = MeshLineConstructor()
 
