@@ -1,8 +1,8 @@
 """This module contains utility functions such as convenient access to
 SciPy linear solvers."""
 
+import sys
 import logging
-from warnings import warn
 from typing import Optional, Union, Tuple, Callable, Dict
 
 import numpy as np
@@ -10,11 +10,16 @@ import scipy.sparse as sp
 import scipy.sparse.csgraph as spg
 import scipy.sparse.linalg as spl
 from numpy import ndarray
-from scipy.sparse import spmatrix
+
+if "pyodide" in sys.modules:
+    from scipy.sparse.base import spmatrix
+else:
+    from scipy.sparse import spmatrix
 
 from skfem.assembly import asm, BilinearForm, LinearForm, DofsView
 from skfem.assembly.basis import AbstractBasis
 from skfem.element import ElementVector
+from skfem.generic_utils import deprecated
 
 
 logger = logging.getLogger(__name__)
@@ -162,13 +167,41 @@ def solver_iter_pcg(**kwargs) -> LinearSolver:
     return solver_iter_krylov(**kwargs)
 
 
+def solver_iter_cg(**kwargs):
+    """Pure Python conjugate gradient solver (for old scipy versions)."""
+
+    def solver(A, b, **solve_time_kwargs):
+        kwargs.update(solve_time_kwargs)
+        maxiters = kwargs['maxiters'] if 'maxiters' in kwargs else 500
+        tol = kwargs['tol'] if 'tol' in kwargs else 1e-10
+        x = b
+        r = b - A.dot(x)
+        p = r
+        rsold = np.dot(r, r)
+        for k in range(maxiters):
+            Ap = A.dot(p)
+            alpha = rsold / np.dot(p, Ap)
+            x = x + alpha * p
+            r = r - alpha * Ap
+            rsnew = np.dot(r, r)
+            if np.sqrt(rsnew) < tol:
+                break
+            p = r + (rsnew / rsold) * p
+            rsold = rsnew
+        if k == maxiters:
+            logger.warning("Iterative solver did not converge.")
+        return x
+
+    return solver
+
+
 # solve and condense
 
 def solve_eigen(A: spmatrix,
                 M: spmatrix,
                 x: Optional[ndarray] = None,
                 I: Optional[ndarray] = None,
-                solver: Optional[Union[EigenSolver]] = None,
+                solver: Optional[EigenSolver] = None,
                 **kwargs) -> Tuple[ndarray, ndarray]:
 
     if solver is None:
@@ -186,7 +219,7 @@ def solve_linear(A: spmatrix,
                  b: ndarray,
                  x: Optional[ndarray] = None,
                  I: Optional[ndarray] = None,
-                 solver: Optional[Union[LinearSolver]] = None,
+                 solver: Optional[LinearSolver] = None,
                  **kwargs) -> ndarray:
 
     if solver is None:
@@ -406,8 +439,8 @@ def penalize(A: spmatrix,
 
     d = Aout.diagonal()
     if epsilon is None:
-        epsilon = 1e-10 / np.linalg.norm(d[D], np.inf)
-    d[D] = 1/epsilon
+        epsilon = 1e-10 / np.linalg.norm(d[D], np.inf).astype(float)
+    d[D] = 1. / epsilon
     Aout.setdiag(d)
 
     if b is None:
@@ -582,6 +615,7 @@ def adaptive_theta(est, theta=0.5, max=None):
         return np.nonzero(theta * max < est)[0]
 
 
+@deprecated("Basis.project")
 def projection(fun,
                basis_to: Optional[AbstractBasis] = None,
                basis_from: Optional[AbstractBasis] = None,
@@ -616,11 +650,11 @@ def projection(fun,
     def mass(u, v, w):
         from skfem.helpers import dot, ddot
         p = 0
-        if len(u.value.shape) == 2:
+        if len(u.shape) == 2:
             p = u * v
-        elif len(u.value.shape) == 3:
+        elif len(u.shape) == 3:
             p = dot(u, v)
-        elif len(u.value.shape) == 4:
+        elif len(u.shape) == 4:
             p = ddot(u, v)
         return p
 
@@ -654,14 +688,13 @@ def projection(fun,
     return solve_linear(M, f)
 
 
+@deprecated("Basis.project")
 def project(fun,
             basis_from: Optional[AbstractBasis] = None,
             basis_to: Optional[AbstractBasis] = None,
             diff: Optional[int] = None,
             I: Optional[ndarray] = None,
             expand: bool = False) -> ndarray:
-    warn("project is deprecated in favor of projection.",
-         DeprecationWarning)
     return projection(
         fun,
         basis_to=basis_to,
